"""Utils related to simple data types"""


def parse_float(float_str, default=None):
    """Parse float.

    Args:
        float_str (str): float as string
        default (, optional): optional value to return of parsing fails

    Return:
        float value

    .. code-block:: python

        >>> from utils.dt import parse_float
        >>> parse_float('1.23')
        1.23
        >>> parse_float('1.23abc')
        None
        >>> parse_float('1.23abc', 'abc')
        'abc'

    """
    try:
        return (float)(float_str)
    except ValueError:
        return default


def parse_int(int_str, default=None):
    """Parse int."""
    try:
        return (int)(int_str)
    except ValueError:
        return default



"""Browser utils."""

from selenium import webdriver
from selenium.webdriver.firefox.options import Options


def open_browser(url):
    """Open brower and return."""
    firefox_profile = webdriver.FirefoxProfile()
    firefox_profile.set_preference('browser.download.folderList', 2)
    firefox_profile.set_preference(
        'browser.download.manager.showWhenStarting',
        False,
    )
    firefox_profile.set_preference('browser.download.dir', '/tmp/')
    firefox_profile.set_preference(
        'browser.helperApps.neverAsk.saveToDisk',
        'text/csv',
    )

    options = Options()
    options.headless = True
    browser = webdriver.Firefox(
        options=options,
        firefox_profile=firefox_profile,
    )
    browser.get(url)
    return browser



"""System utils."""
import os
import time

import subprocess
import psutil


def log_metrics():
    """Log system metrics.

    .. code-block:: python

        >>> from utils import sysx
        >>> print(sysx.log_metrics())
        {"ut": 1620724794.43984, "pid": 15129,
            "cpu_percent": 16.3, "vm_percent": 65.7}

    Note:
        Needs psutil

        .. code-block:: bash

            pip install psutil
    """
    log = {
        'ut': time.time(),
        'pid': os.getpid(),
        'cpu_percent': psutil.cpu_percent(),
        'vm_percent': psutil.virtual_memory().percent,
    }
    return log


def run(cmd):
    """Run commands.

    .. code-block:: python

        >>> from utils import sysx
        >>> print(sysx.run('echo "hello"'))
        ['hello']

    """
    process = subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=True)
    (output, _) = process.communicate()
    return output.decode().split('\n')[:-1]


def str_color(output_str, color_code=31):
    """Wrap standard print command, to add color."""
    color_cmd = '\033[0;%dm' % (color_code)
    end_cmd = '\033[0m'
    return '%s%s%s' % (color_cmd, str(output_str), end_cmd)



"""Utils for caching functions."""
import json

from functools import wraps

from utils.cache_impl import _Cache

CACHE_DIR = '/tmp/cache'
CACHE_DEFAULT_TIMEOUT = 60


def cache(cache_name, timeout=CACHE_DEFAULT_TIMEOUT):
    """Wrap class Cache as decorator."""

    def cache_inner(func):

        @wraps(func)
        def cache_inner_inner(*args, **kwargs):
            cache_key = json.dumps({
                'cache_name': cache_name,
                'function_name': func.__name__,
                'kwargs': kwargs,
                'args': args,
            })

            def fallback():
                return func(*args, **kwargs)

            return _Cache(cache_name, timeout).get(cache_key, fallback)

        return cache_inner_inner
    return cache_inner
